@@ -1,4 +1,4 @@
-# $Id: ODBC.pm 11768 2008-09-12 13:00:28Z mjevans $
+# $Id: ODBC.pm 12450 2009-01-16 18:47:37Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
@@ -8,9 +8,11 @@
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
 
+## no critic (ProhibitManyArgs)
+
 require 5.006;
 
-$DBD::ODBC::VERSION = '1.17';
+$DBD::ODBC::VERSION = '1.18';
 
 {
     package DBD::ODBC;
@@ -21,16 +23,16 @@ $DBD::ODBC::VERSION = '1.17';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 11768 2008-09-12 13:00:28Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 12450 2009-01-16 18:47:37Z mjevans $, 13,2);
 
     require_version DBI 1.21;
 
     bootstrap DBD::ODBC $VERSION;
 
-    $err = 0;		# holds error code   for DBI::err
-    $errstr = "";	# holds error string for DBI::errstr
+    $err = 0;           # holds error code   for DBI::err
+    $errstr = "";       # holds error string for DBI::errstr
     $sqlstate = "00000";
-    $drh = undef;	# holds driver handle once initialised
+    $drh = undef;       # holds driver handle once initialised
 
     sub parse_trace_flag {
         my ($class, $name) = @_;
@@ -74,6 +76,7 @@ $DBD::ODBC::VERSION = '1.17';
     use strict;
     use warnings;
 
+    ## no critic (ProhibitBuiltinHomonyms)
     sub connect {
 	my $drh = shift;
 	my($dbname, $user, $auth, $attr)= @_;
@@ -90,10 +93,11 @@ $DBD::ODBC::VERSION = '1.17';
 	# Call ODBC logon func in ODBC.xs file
 	# and populate internal handle data.
 
-	DBD::ODBC::db::_login($this, $dbname, $user, $auth, $attr) or return undef;
+	DBD::ODBC::db::_login($this, $dbname, $user, $auth, $attr) or return;
 
 	$this;
     }
+    ## use critic
 
 }
 
@@ -122,7 +126,8 @@ $DBD::ODBC::VERSION = '1.17';
                 odbc_out_connect_string => undef,
                 odbc_version => undef,
                 odbc_err_handler => undef,
-                odbc_putdata_start => undef # sth and dbh
+                odbc_putdata_start => undef, # sth and dbh
+                odbc_column_display_size => undef # sth and dbh
                };
     }
 
@@ -139,7 +144,7 @@ $DBD::ODBC::VERSION = '1.17';
 	# and populate internal handle data.
 
 	DBD::ODBC::st::_prepare($sth, $statement, @attribs)
-	    or return undef;
+	    or return;
 
 	$sth;
     }
@@ -155,7 +160,7 @@ $DBD::ODBC::VERSION = '1.17';
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
 
 	_columns($dbh,$sth, $catalog, $schema, $table, $column)
-	    or return undef;
+	    or return;
 
 	$sth;
     }
@@ -171,7 +176,7 @@ $DBD::ODBC::VERSION = '1.17';
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
 
 	_columns($dbh,$sth, $catalog, $schema, $table, $column)
-	    or return undef;
+	    or return;
 
 	$sth;
     }
@@ -197,7 +202,7 @@ $DBD::ODBC::VERSION = '1.17';
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables" });
 
 	DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
-	      or return undef;
+	      or return;
 	$sth;
     }
 
@@ -211,10 +216,9 @@ $DBD::ODBC::VERSION = '1.17';
        $schema = "" if (!$schema);
        $table = "" if (!$table);
        DBD::ODBC::st::_primary_keys($dbh,$sth, $catalog, $schema, $table )
-	     or return undef;
+	     or return;
        $sth;
     }
-
     sub statistics_info {
        my ($dbh, $catalog, $schema, $table, $unique, $quick ) = @_;
 
@@ -229,7 +233,7 @@ $DBD::ODBC::VERSION = '1.17';
 
        DBD::ODBC::st::_statistics($dbh, $sth, $catalog, $schema, $table,
                                  $unique, $quick)
-	     or return undef;
+	     or return;
        $sth;
     }
 
@@ -245,7 +249,7 @@ $DBD::ODBC::VERSION = '1.17';
        $fkcatalog = "" if (!$fkcatalog);
        $fkschema = "" if (!$fkschema);
        $fktable = "" if (!$fktable);
-       _GetForeignKeys($dbh, $sth, $pkcatalog, $pkschema, $pktable, $fkcatalog, $fkschema, $fktable) or return undef;
+       _GetForeignKeys($dbh, $sth, $pkcatalog, $pkschema, $pktable, $fkcatalog, $fkschema, $fktable) or return;
        $sth;
     }
 
@@ -298,9 +302,9 @@ $DBD::ODBC::VERSION = '1.17';
 	    $DBD::ODBC::sqlstate = "00000";
 	    return 1 if $ok;
 	}
-	return 1 if $state eq 'S0002';	# Base table not found
+        return 1 if $state eq 'S0002';  # Base table not found
  	return 1 if $state eq '42S02';  # Base table not found.Solid EE v3.51
-	return 1 if $state eq 'S0022';	# Column not found
+        return 1 if $state eq 'S0022';  # Column not found
 	return 1 if $state eq '37000';  # statement could not be prepared (19991011, JLU)
 	# return 1 if $state eq 'S1000';  # General Error? ? 5/30/02, JLU.  This is what Openlink is returning
 	# We assume that any other error means the database
@@ -317,7 +321,7 @@ $DBD::ODBC::VERSION = '1.17';
 	# handle SQL_DRIVER_HSTMT, SQL_DRIVER_HLIB and
 	# SQL_DRIVER_HDESC specially
 	if ($item == 5 || $item == 135 || $item == 76) {
-	   return undef;
+	   return;
 	}
 	return _GetInfo($dbh, $item);
     }
@@ -325,6 +329,7 @@ $DBD::ODBC::VERSION = '1.17';
     # new override of do method provided by Merijn Broeren
     # this optimizes "do" to use SQLExecDirect for simple
     # do statements without parameters.
+    ## no critic (ProhibitBuiltinHomonyms)
     sub do {
         my($dbh, $statement, $attr, @params) = @_;
         my $rows = 0;
@@ -347,7 +352,7 @@ $DBD::ODBC::VERSION = '1.17';
         }
         return $rows
     }
-
+    ## use critic
     #
     # can also be called as $dbh->func($sql, ExecDirect);
     # if, for some reason, there are compatibility issues
@@ -374,11 +379,11 @@ $DBD::ODBC::VERSION = '1.17';
     # See the ODBC documentation for more information about this call.
     #
     sub GetStatistics {
-			my ($dbh, $Catalog, $Schema, $Table, $Unique) = @_;
-			# create a "blank" statement handle
-			my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLStatistics" });
-			_GetStatistics($dbh, $sth, $Catalog, $Schema, $Table, $Unique) or return undef;
-			$sth;
+        my ($dbh, $Catalog, $Schema, $Table, $Unique) = @_;
+        # create a "blank" statement handle
+        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLStatistics" });
+        _GetStatistics($dbh, $sth, $Catalog, $Schema, $Table, $Unique) or return;
+        $sth;
     }
 
     # Call the ODBC function SQLForeignKeys
@@ -386,11 +391,11 @@ $DBD::ODBC::VERSION = '1.17';
     # See the ODBC documentation for more information about this call.
     #
     sub GetForeignKeys {
-			my ($dbh, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) = @_;
-			# create a "blank" statement handle
-			my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLForeignKeys" });
-			_GetForeignKeys($dbh, $sth, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) or return undef;
-			$sth;
+        my ($dbh, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) = @_;
+        # create a "blank" statement handle
+        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLForeignKeys" });
+        _GetForeignKeys($dbh, $sth, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) or return;
+        $sth;
     }
 
     # Call the ODBC function SQLPrimaryKeys
@@ -398,11 +403,11 @@ $DBD::ODBC::VERSION = '1.17';
     # See the ODBC documentation for more information about this call.
     #
     sub GetPrimaryKeys {
-			my ($dbh, $Catalog, $Schema, $Table) = @_;
-			# create a "blank" statement handle
-			my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLPrimaryKeys" });
-			_GetPrimaryKeys($dbh, $sth, $Catalog, $Schema, $Table) or return undef;
-			$sth;
+        my ($dbh, $Catalog, $Schema, $Table) = @_;
+        # create a "blank" statement handle
+        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLPrimaryKeys" });
+        _GetPrimaryKeys($dbh, $sth, $Catalog, $Schema, $Table) or return;
+        $sth;
     }
 
     # Call the ODBC function SQLSpecialColumns
@@ -413,7 +418,7 @@ $DBD::ODBC::VERSION = '1.17';
 	my ($dbh, $Identifier, $Catalog, $Schema, $Table, $Scope, $Nullable) = @_;
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLSpecialColumns" });
-	_GetSpecialColumns($dbh, $sth, $Identifier, $Catalog, $Schema, $Table, $Scope, $Nullable) or return undef;
+	_GetSpecialColumns($dbh, $sth, $Identifier, $Catalog, $Schema, $Table, $Scope, $Nullable) or return;
 	$sth;
     }
 
@@ -422,7 +427,7 @@ $DBD::ODBC::VERSION = '1.17';
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
 	# print "SQL Type is $sqltype\n";
-	_GetTypeInfo($dbh, $sth, $sqltype) or return undef;
+	_GetTypeInfo($dbh, $sth, $sqltype) or return;
 	$sth;
     }
 
@@ -430,7 +435,7 @@ $DBD::ODBC::VERSION = '1.17';
 	my ($dbh, $sqltype) = @_;
 	$sqltype = DBI::SQL_ALL_TYPES unless defined $sqltype;
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
-	_GetTypeInfo($dbh, $sth, $sqltype) or return undef;
+	_GetTypeInfo($dbh, $sth, $sqltype) or return;
 	my $info = $sth->fetchall_arrayref;
 	unshift @$info, {
 	    map { ($sth->{NAME}->[$_] => $_) } 0..$sth->{NUM_OF_FIELDS}-1
@@ -454,11 +459,12 @@ $DBD::ODBC::VERSION = '1.17';
                 odbc_force_rebind => undef, # sth & dbh
                 odbc_async_exec => undef, # sth and dbh
                 odbc_query_timeout => undef, # sth and dbh
-                odbc_putdata_start => undef # sth and dbh
+                odbc_putdata_start => undef, # sth and dbh
+                odbc_column_display_size => undef # sth and dbh
                };
     }
 
-    sub ColAttributes {		# maps to SQLColAttributes
+    sub ColAttributes { # maps to SQLColAttributes
 	my ($sth, $colno, $desctype) = @_;
 	# print "before ColAttributes $colno\n";
 	my $tmp = _ColAttributes($sth, $colno, $desctype);
@@ -493,7 +499,7 @@ See L<DBI> for more information.
 =head2 Change log and FAQs
 
 Please note that the change log has been moved to
-DBD::ODBC::Changes.pm. To access this documentation, use
+DBD::ODBC::Changes. To access this documentation, use
 C<perldoc DBD::ODBC::Changes>.
 
 The FAQs have also moved to DBD::ODBC::FAQ.pm. To access the FAQs use
@@ -560,9 +566,9 @@ do - see L</CPAN Testers Reporting>.
 =item Report bugs
 
 If you find what you believe is a bug then enter it into the
-L<http://rt.cpan.org> system. Where possible include code which reproduces the
-problem including any schema required and the versions of software you
-are using.
+L<http://rt.cpan.org/Dist/Display.html?Name=DBD-ODBC> system. Where
+possible include code which reproduces the problem including any
+schema required and the versions of software you are using.
 
 If you are unsure whether you have found a bug report it anyway or
 post it to the dbi-users mailing list.
@@ -607,7 +613,7 @@ DBI specification.
 
 DBI documents the C<ReadOnly> attribute as being settleable and
 retrievable on connection and statement handles. In ODBC setting
-ReadOnly to true causes the connection attribute SQL_ATTR_ACCESS_MODE
+ReadOnly to true causes the connection attribute C<SQL_ATTR_ACCESS_MODE>
 to be set to C<SQL_MODE_READ_ONLY> and setting it to false will set the
 access mode to C<SQL_MODE_READ_WRITE> (which is the default in ODBC).
 
@@ -666,9 +672,11 @@ about this.
 =head3 odbc_async_exec
 
 Allow asynchronous execution of queries.  This causes a spin-loop
-(with a small "sleep") until the SQL is complete.  This is useful,
-however, if you want the error handling and asynchronous messages (see
-the L</odbc_err_handler> and F<t/20SQLServer.t> for an example of this.
+(with a small "sleep") until the ODBC API being called is complete
+(i.e., while the ODBC API returns C<SQL_STILL_EXECUTING>).  This is
+useful, however, if you want the error handling and asynchronous
+messages (see the L</odbc_err_handler> and F<t/20SQLServer.t> for an
+example of this.
 
 =head3 odbc_query_timeout
 
@@ -681,7 +689,7 @@ C<SQL_ATTR_LOGIN_TIMEOUT> and C<SQL_ATTR_CONNECTION_TIMEOUT>. Add
 
   { odbc_query_timeout => 30 }
 
-to your connect, set on the dbh before creating a statement or
+to your connect, set on the C<dbh> before creating a statement or
 explicitly set it on your statement handle. The odbc_query_timeout on
 a statement is inherited from the parent connection.
 
@@ -690,7 +698,7 @@ explicitly and the default of 0 (no time out) is implemented by the
 ODBC driver and not DBD::ODBC.
 
 Note that some ODBC drivers implement a maximum query timeout value
-and will limit time outs set above their maximum. You may see a
+and will limit timeouts set above their maximum. You may see a
 warning if your time out is capped by the driver but there is
 currently no way to retrieve the capped value back from the driver.
 
@@ -700,16 +708,27 @@ See F<t/20SqlServer.t> for an example.
 
 =head3 odbc_putdata_start
 
-odbc_putdata_start defines the size at which DBD::ODBC uses C<SQLPutData>
-and C<SQLParamData> to send larger objects to the database instead of
-simply binding them as normal with C<SQLBindParameter>. It is mostly a
-placeholder for future changes allowing chunks of data to be sent to
-the database and there is little reason for anyone to change it
-currently.
+C<odbc_putdata_start> defines the size at which DBD::ODBC uses
+C<SQLPutData> and C<SQLParamData> to send larger objects to the
+database instead of simply binding them as normal with
+C<SQLBindParameter>. It is mostly a placeholder for future changes
+allowing chunks of data to be sent to the database and there is little
+reason for anyone to change it currently.
 
 The default for odbc_putdata_start is 32768 because this value was
 hard-coded in DBD::ODBC until 1.16_1.
 
+=head3 odbc_column_display_size
+
+If you ODBC driver does not support the SQL_COLUMN_DISPLAY_SIZE and
+SQL_COLUMN_LENGTH attributes to SQLColAtrributes then DBD::ODBC does
+not know how big the column might be. odbc_column_display_size sets
+the default value for the column size when retrieving column data
+where the size cannot be determined.
+
+The default for odbc_column_display_size is 2001 because this value was
+hard-coded in DBD::ODBC until 1.17_3.
+
 =head2 Private connection attributes
 
 =head3 odbc_err_handler
@@ -755,7 +774,7 @@ C<SQL_ROWSET_SIZE> attribute patch from Andrew Brown
   > SQL_ROWSET_SIZE as db handle option.
   >
   > The purpose to my madness is simple. SqlServer (7 anyway) by default
-  > supports only one select statement at once (using std ODBC cursors).
+  > supports only one select statement at once (using standard ODBC cursors).
   > According to the SqlServer documentation you can alter the default
   > setting of three values to force the use of server cursors - in
   > which case multiple selects are possible.
@@ -773,7 +792,8 @@ C<SQL_ROWSET_SIZE> attribute patch from Andrew Brown
   >
 
 In versions of SQL Server 2008 and later see "Multiple Active
-Statements (MAS)" in the DBD::ODBC::FAQ instead of using this attribute.
+Statements (MAS)" in the DBD::ODBC::FAQ instead of using this
+attribute.
 
 =head3 odbc_exec_direct
 
@@ -847,11 +867,6 @@ the connect method and a lot more. See L</Unicode>.
 
 When odbc_has_unicode is 1, DBD::ODBC will:
 
-=head3 odbc_out_connect_string
-
-After calling the connect method this will be the ODBC driver's
-out connection string - see documentation on SQLDriverConnect.
-
 =over
 
 =item bind columns the database declares as wide characters as SQL_Wxxx
@@ -866,21 +881,29 @@ a wide character (or where the parameter type is explicitly set to a
 wide type - SQL_Wxxx) can be UTF8 in Perl and will be mapped to UTF16
 before passing to the driver.
 
+=item SQL
+
+SQL passed to the C<prepare> or C<do> methods which has the UTF8 flag set
+will be converted to UTF16 before being passed to the ODBC APIs C<SQLPrepare>
+or C<SQLExecDirect>.
+
+=item connection strings
+
+Connection strings passed to the C<connect> method will be converted
+to UTF16 before being passed to the ODBC API C<SQLDriverConnectW>. This happens
+irrespective of whether the UTF8 flag is set on the perl connect strings
+because unixODBC requires an application to call SQLDriverConnectW to indicate
+it will be calling the wide ODBC APIs.
+
 =back
 
 NOTE: You will need at least Perl 5.8.1 to use UNICODE with DBD::ODBC.
 
-NOTE: At this time SQL statements are still treated as native encoding
-i.e. DBD::ODBC does not call SQLPrepareW with UNICODE strings. If you
-need a unicode constant in an SQL statement, you have to pass it as
-parameter or use SQL functions to convert your constant from native
-encoding to Unicode.
-
 NOTE: Binding of unicode output parameters is coded but untested.
 
 NOTE: When building DBD::ODBC on Windows ($^O eq 'MSWin32') the
 WITH_UNICODE macro is automatically added. To disable specify -nou as
-an argument to Makefile.PL (e.g. nmake Makefile.PL -nou). On non-Windows
+an argument to Makefile.PL (e.g. C<perl Makefile.PL -nou>). On non-Windows
 platforms the WITH_UNICODE macro is B<not> enabled by default and to enable
 you need to specify the -u argument to Makefile.PL. Please bare in mind
 that some ODBC drivers do not support SQL_Wxxx columns or parameters.
@@ -891,6 +914,11 @@ untested.  Let me know how you get on with it.
 UNICODE support in ODBC Drivers differs considerably. Please read the
 README.unicode file for further details.
 
+=head3 odbc_out_connect_string
+
+After calling the connect method this will be the ODBC driver's
+out connection string - see documentation on SQLDriverConnect.
+
 =head3 odbc_version
 
 This was added prior to the move to ODBC 3.x to allow the caller to
@@ -909,14 +937,14 @@ now 3.x, this can be used to force 2.x behavior via something like: my
 Use this attribute to determine if there are more result sets
 available.  SQL Server supports this feature.  Use this as follows:
 
-do {
-   my @row;
-   while (@row = $sth->fetchrow_array()) {
-      # do stuff here
-   }
-} while ($sth->{odbc_more_results});
+  do {
+     my @row;
+     while (@row = $sth->fetchrow_array()) {
+        # do stuff here
+     }
+  } while ($sth->{odbc_more_results});
 
-Note that with multiple result sets and output parameters (i.e. using
+Note that with multiple result sets and output parameters (i.e,. using
 bind_param_inout, don't expect output parameters to be bound until ALL
 result sets have been retrieved.
 
@@ -924,7 +952,7 @@ result sets have been retrieved.
 
 You use DBD::ODBC private functions like this:
 
-  $dbh->func(arg, private_function_name);
+  $dbh->func(arg, private_function_name, @args);
 
 =head3 GetInfo
 
@@ -936,7 +964,7 @@ e.g.
 
   $value = $dbh->func(6, 'GetInfo');
 
-which returns the SQL_DRIVER_NAME.
+which returns the C<SQL_DRIVER_NAME>.
 
 This function returns a scalar value, which can be a numeric or string
 value depending on the information value requested.
@@ -962,23 +990,23 @@ This function returns a DBI statement handle for the SQLGetTypeInfo
 result-set containing many columns of type attributes (see ODBC
 specification).
 
-NOTE: It is VERY important that the use DBI includes the
-qw(:sql_types) so that values like SQL_VARCHAR are correctly
+NOTE: It is VERY important that the C<use DBI> includes the
+C<qw(:sql_types)> so that values like SQL_VARCHAR are correctly
 interpreted.  This "imports" the sql type names into the program's
-name space.  A very common mistake is to forget the qw(:sql_types) and
-obtain strange results.
+name space.  A very common mistake is to forget the C<qw(:sql_types)>
+and obtain strange results.
 
 =head3 GetFunctions
 
 This function maps to the ODBC SQLGetFunctions API which returns
 information on whether a function is supported by the ODBC driver.
 
-The argument should be SQL_API_ALL_FUNCTIONS (0) for all functions or
-a valid ODBC function number (e.g. SQL_API_SQLDESCRIBEPARAM which is
-58). See ODBC specification or examine your sqlext.h and sql.h header
-files for all the SQL_API_XXX macros.
+The argument should be C<SQL_API_ALL_FUNCTIONS> (0) for all functions
+or a valid ODBC function number (e.g. C<SQL_API_SQLDESCRIBEPARAM>
+which is 58). See ODBC specification or examine your sqlext.h and
+sql.h header files for all the SQL_API_XXX macros.
 
-If called with SQL_API_ALL_FUNCTIONS (0), then a 100 element array is
+If called with C<SQL_API_ALL_FUNCTIONS> (0), then a 100 element array is
 returned where each element will contain a '1' if the ODBC function with
 that SQL_API_XXX index is supported or '' if it is not.
 
